from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

from django_dropimages import settings as di_settings


class DropImagesConfig(AppConfig):
    name = 'django_dropimages'
    verbose_name = _("Django DropImages")

    def ready(self):
        if di_settings.PATCH_SETTINGS:
            di_settings.patch_all()


from __future__ import absolute_import, unicode_literals
from importlib import import_module

from django.conf import settings

# Always import this module as follows:
# from django_dropimages import settings [as di_settings]

CONFIG_DEFAULTS = {
    'DICT_DEFAULT_MESSAGE': 'To upload drop here files, a directory or click.',

    'SHOW_ID_ON_COMPLETE': None,
    'GALLERY_FIELD_ID': None,
    'UPLOAD_URL': None,
    'DELETE_URL': None,

    'DROPIMAGE_MODEL': None,
    'DROPIMAGE_FIELD': None,
}

USER_CONFIG = getattr(settings, 'DROP_IMAGES_CONFIG', {})

CONFIG = CONFIG_DEFAULTS.copy()
CONFIG.update(USER_CONFIG)

PATCH_SETTINGS = getattr(settings, 'DROP_IMAGES_PATCH_SETTINGS', True)


# The following functions can monkey-patch settings automatically. Several
# imports are placed inside functions to make it safe to import this module.


def patch_root_urlconf():
    from django.conf.urls import include, url
    from django.core.urlresolvers import clear_url_caches, reverse, NoReverseMatch
    import django_dropimages
    try:
        reverse('djdropimages:render_dropzone')
    except NoReverseMatch:
        urlconf_module = import_module(settings.ROOT_URLCONF)
        urlconf_module.urlpatterns = [
            url(r'^__dropimages__/', include(django_dropimages.urls)),
        ] + urlconf_module.urlpatterns
        clear_url_caches()


def patch_all():
    patch_root_urlconf()



from django.conf import settings
from django.db import models

from django_dropimages import settings as di_settings


class DropimagesGallery(models.Model):
    gallery_identifier = models.CharField(max_length=36)
    creation_timestamp = models.DateTimeField(auto_now_add=True)
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)

# if no custom image models is present I load my own
if not di_settings.CONFIG['DROPIMAGE_MODEL']:
    class DropimagesImage(models.Model):
        dropimages_gallery = models.ForeignKey('django_dropimages.DropimagesGallery', related_name='images')
        dropimages_original_filename = models.CharField(max_length=256)
        image = models.ImageField(upload_to='%y/%m/%d')



import uuid

from django import template
from django.core.urlresolvers import reverse_lazy
from django.template import Context

from django_dropimages import settings as di_settings

register = template.Library()


@register.tag(name="drop_images_js")
def drop_images_js(parser, token):
    return DropImagesJSNode()


class DropImagesJSNode(template.Node):
    def render(self, context):
        t = template.loader.get_template('django_dropimages/dropimagesjs.html')
        return t.render(Context({
            'gallery_id': uuid.uuid4(),

            'dict_default_message': di_settings.CONFIG['DICT_DEFAULT_MESSAGE'],
            'gallery_field_id': di_settings.CONFIG['GALLERY_FIELD_ID'],

            'id_to_show': di_settings.CONFIG['SHOW_ID_ON_COMPLETE'] or None,
            'upload_url': di_settings.CONFIG['UPLOAD_URL'] or reverse_lazy('djdropimages:upload'),
            'delete_url': di_settings.CONFIG['DELETE_URL'] or reverse_lazy('djdropimages:delete'),
        }))


@register.tag(name="drop_images")
def drop_images(parser, token):
    return DropImagesNode()


class DropImagesNode(template.Node):
    def render(self, context):
        t = template.loader.get_template('django_dropimages/dropimages.html')
        return t.render(Context({}))



from django.shortcuts import get_object_or_404
from django.views.generic import View
from braces.views import CsrfExemptMixin, JSONResponseMixin

from django_dropimages import settings as di_settings
from .models import DropimagesGallery

from django.apps import apps


class UploadView(CsrfExemptMixin, JSONResponseMixin, View):
    def post(self, request, *args, **kwargs):
        owner = request.user if not request.user.is_anonymous() else None
        image = request.FILES['file']
        gallery, _ = DropimagesGallery.objects.get_or_create(gallery_identifier=request.GET['gallery_id'],
                                                             owner=owner)
        image_klass = apps.get_model(di_settings.CONFIG['DROPIMAGE_MODEL'] or 'django_dropimages.DropimagesImage')
        kwargs_dict = {
            'dropimages_gallery': gallery,
            'dropimages_original_filename': image._name,
            di_settings.CONFIG['DROPIMAGE_FIELD'] or 'image': image
        }
        image_klass.objects.create(**kwargs_dict)
        return self.render_json_response({
            'gallery_identifier': request.GET['gallery_id'],
            'gallery_pk': gallery.pk,
            'file_name': image._name,
        })


class DeleteView(CsrfExemptMixin, JSONResponseMixin, View):
    def get(self, request, *args, **kwargs):
        owner = request.user if not request.user.is_anonymous() else None
        gallery = get_object_or_404(DropimagesGallery, gallery_identifier=request.GET['gallery_id'], owner=owner)
        image_klass = apps.get_model(di_settings.CONFIG['DROPIMAGE_MODEL'] or 'django_dropimages.DropimagesImage')
        di = get_object_or_404(image_klass, dropimages_gallery=gallery,
                               dropimages_original_filename=request.GET['original_filename'])
        di.delete()
        return self.render_json_response({})



from __future__ import absolute_import, unicode_literals

__all__ = ['VERSION']


try:
    import pkg_resources
    VERSION = pkg_resources.get_distribution('django-dropimages').version
except Exception:
    VERSION = 'unknown'


# Code that discovers files or modules in INSTALLED_APPS imports this module.
# Reference URLpatterns with a string to avoid the risk of circular imports.

urls = 'django_dropimages.urls', 'djdropimages', 'djdropimages'


default_app_config = 'django_dropimages.apps.DropImagesConfig'



from io import open

from setuptools import find_packages, setup

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
    print "README.md succesfully converted"
except(IOError, ImportError):
    long_description = open('README.md').read()

setup(
    name='django-dropimages',
    version='0.1.4',
    description='Uses Dropzone.js and django-braces to simplify the upload of multiple images into a collection object.',
    long_description=long_description,
    author='Simone Cittadini',
    author_email='simone@sig-c.com',
    url='https://github.com/sittizen/django_dropimages',
    download_url='https://pypi.python.org/pypi/django-dropimages',
    license='MIT',
    packages=find_packages(exclude=('tests.*', 'tests', 'example')),
    install_requires=[
        'Django>=1.7',
        'django-braces<1.9',
        'Pillow'
    ],
    include_package_data=True,
    zip_safe=False,                 # because we're including static files
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
